#!/bin/ash
# add/remove/change drive-icon to ROX-Filer pinboard
# (C) James Budiono 2012, 2013
# License: GNU GPL Version 3 or later
#
# Version 1.0 - Support multiple X desktops
# Version 2.0 - re-written to back-end/front-end to make it easier to port to another desktop
#               (front-end is fatdog-drive-icon-start-v2.sh)
#             - multiple X desktop supports is now handled by front-end automatically
#
# called from udev where ACTION=add/remove/change and SUBSYSTEM=block, executed as root, no X
# uses uses ACTION, DEVNAME, DEVPATH environments

### system configuration - no need to change
export PATH=/sbin:/bin:/usr/sbin:/usr/bin
TEMPLATE_DRIVE_ICON_ROOT=/tmp/fatdog-drive-icons
UDEV_DRIVE_ICON_ROOT=/tmp/fatdog-drive-icon-udev
EVENT_COUNTER=$UDEV_DRIVE_ICON_ROOT/counter
F_MUTEX=$UDEV_DRIVE_ICON_ROOT/busy
MUTEX_TIMEOUT=50 									# 5 seconds (unit is 1/10 seconds)

DEBUG_FILE=$UDEV_DRIVE_ICON_ROOT/log
DEBUG=

### run-time variables
DRIVE_ICON_DIR=

# get the rox functions
. /usr/sbin/fatdog-drive-icon-roxlib.sh # create_rox_app, create_rox_icon_file


#################### utilities #####################
# output $@ to debug file
log_debug() {
	[ "$DEBUG" ] && echo "$@" >> $DEBUG_FILE
}

### simple, stupid, but quick way to determine drive type 
# $1-drivename $2-devpath returns DRIVE_TYPE ("primary-secondary")
guess_drive_type() {
	DRIVE_TYPE=any
	case $1 in 
		fd*) DRIVE_TYPE=floppy ;;
		sr*) DRIVE_TYPE=optical ;;
		mmc*) DRIVE_TYPE=card ;;
	esac
	case $2 in
		*usb*) DRIVE_TYPE="$DRIVE_TYPE-usb"
	esac
}

### get summary for mouseover tip window display
# $DEVNAME, $1-FS_TYPE returns SUMMARY
get_summary() {
	SUMMARY=""
	[ $SHOW_VOLUME_LABEL -a "$1" -a "$1" != "unknown" ] && SUMMARY=$(busybox blkid $DEVNAME | sed -n "\_^${DEVNAME}:_ {"'/LABEL/ {s/.*LABEL="\([^"]*\)".*/\1/; p}}')
	#[ $SHOW_VOLUME_LABEL ] && SUMMARY=$(disktype $DEVNAME | sed -n '/Volume name/ { s/.*\(".*"\).*/\1/; p}')
	read p < /sys/$DEVPATH/size
	p=$(( p / 2048 )) # convert to MiB (= 1024 KB) assuming 512-byte units
	[ $p -ge 2048 ] && SUMMARY="$SUMMARY $(( p / 1024))GiB" || SUMMARY="$SUMMARY ${p}MiB" # convert to GiB (=1024 MiB) if more than 2GiB
	[ "$1" ] && SUMMARY="$SUMMARY $1"
}


#################### mutex management #####################

# get mutex
get_mutex() {
	local timeout=$MUTEX_TIMEOUT
	while ! mkdir $F_MUTEX; do
		sleep 0.1; 
		case $timeout in 
			0) break ;;
			*) timeout=$((timeout - 1)) ;;
		esac
	done
	log_debug lock $DRIVENAME
}

# release mutex
release_mutex() {
	rm -rf $F_MUTEX
	log_debug unlock $DRIVENAME
}


#################### main udev handlers here #####################

### add handler - add icons to rox
udev_add_handler() {
	PROCEED= FS_TYPE=""
	
	guess_drive_type $DRIVENAME $DEVPATH 	# returns DRIVE_TYPE	
	case $DRIVE_TYPE in
		floppy*) PROCEED=yes ;;									# floppy - always display
		optical*) ! { disktype $DEVNAME 2>&1 1>/dev/null | grep -qi "no medium"; }  && # optical - check if there is disc inside
				  PROCEED=yes && fatdog-drive-icon-poll-optical.sh $DEVNAME ;;	# and start polling - because kernel poll doesn't work :(
		*)	FS_TYPE=$(guess_fstype $DEVNAME)					# others - show icon if we recognise the filesystem
			case "$FS_TYPE" in 	  				
				unknown|""|Usage*|swap) log_debug "$DEVNAME filesystem unknown" ;; # unknown or swap - do nothing
				*) PROCEED=yes;;
			esac ;;
	esac

	if [ $PROCEED ]; then
		mkdir -p $DRIVE_ICON_DIR
		get_summary $FS_TYPE # returns SUMMARY
		create_rox_app $DRIVE_ICON_DIR $DRIVE_TYPE $DEVNAME "$SUMMARY"
	fi
}

udev_change_handler() {
	
	case $DRIVENAME in 
	sr*)  if [ -e $DRIVE_ICON_DIR ] ; then  #If change action for optical, assume eject if icon exists. 
		  rm -rf $DRIVE_ICON_DIR
		  ACTION="remove"
		  else
		  rm -rf $DRIVE_ICON_DIR
	      udev_add_handler 
		  fi ;;
	 *)   rm -rf $DRIVE_ICON_DIR
	      udev_add_handler ;;
	esac
}

### remove and change handler - directly implemented, see below

#################### main #####################
#DEVPATH=/devices/pci0000:00/0000:00:1f.2/host1/target1:0:0/1:0:0:0/block/sr0
#DEVNAME=/dev/sr0

DRIVENAME=${DEVNAME##*/}
case $DRIVENAME in loop*) exit ;; esac	# ignore these devices

[ ! -e $UDEV_DRIVE_ICON_ROOT ] && mkdir -p $UDEV_DRIVE_ICON_ROOT
DRIVE_ICON_DIR=$UDEV_DRIVE_ICON_ROOT/$DRIVENAME
get_mutex

# create rox-apps
case $ACTION in
	add)	udev_add_handler ;;
	remove)	rm -rf $DRIVE_ICON_DIR ;;
	change) udev_change_handler ;;
esac

# notify
! [ -e $EVENT_COUNTER ] && echo 0 > $EVENT_COUNTER
read counter < $EVENT_COUNTER

for p in ${TEMPLATE_DRIVE_ICON_ROOT}*; do
	log_debug notify $p $ACTION $DRIVENAME
	echo $ACTION $DRIVENAME > $p/event$counter
	echo $counter > $p/update
done

echo $(( counter + 1 )) > $EVENT_COUNTER
release_mutex
